function [ result ] = extractKmerContribs( maxOrder, seqLen, opts);
% [ result ] = extractKmerContribs( maxOrder, seqLen, opts);

result=[];
alphabetSize = 4;

if( ~exist('opts','var') )
  opts = [];
end;

if ~isfield(opts, 'consensus')
	opts = setDefault( opts, 'consensus', 0);
end

if ~isfield(opts, 'poim')
	opts = setDefault( opts, 'poim', 0);
end

if ~isfield(opts, 'w')
	opts = setDefault( opts, 'w', 0);
end

if ~isfield(opts, 'hull')
	opts = setDefault( opts, 'hull', 0);
end

if ~isfield(opts, 'weightmass')
	opts = setDefault( opts, 'weightmass', 0);
end

if opts.consensus
	disp('computing consensus')
	result.consensus_seq = sg('get_WD_consensus'); 
end

if opts.w
	disp('computing w')
	x=sg( 'get_kernel_optimization', maxOrder );
	[result.w, result.max_w] = reshape_normalize_contribs(x, maxOrder, seqLen, opts.distribution);
	clear x;

	if opts.hull,
		[result.w_hull, result.w_kmerdiversity]=compute_hull_and_diversity(result.w, maxOrder);
	end

	if opts.weightmass
		[result.w_weightmass, result.w_totalmass ] =compute_weight_mass(result.w, maxOrder, seqLen);
	end
end

if opts.poim
	disp('computing poim')
	if ~isfield(opts, 'distribution')
		error('poim computation requires distribution');
	end
	x = sg( 'compute_poim_wd', maxOrder, opts.distribution );
	[result.poim, result.max_poim, result.diff_poim] = reshape_normalize_contribs(x, maxOrder, seqLen, opts);
	clear x;

	if opts.hull,
		[result.poim_hull, result.poim_kmerdiversity]=compute_hull_and_diversity(result.poim, maxOrder);
	end

	if opts.weightmass
		[result.poim_weightmass, result.poim_totalmass ] = compute_weight_mass(result.poim, maxOrder, seqLen);
	end
end

return

%%
function [mass,total] = compute_weight_mass(C, maxOrder, seqLen)
mass=zeros(maxOrder, seqLen);
total=zeros(1, seqLen);
for i=1:maxOrder,
	mass(i,:)=sum(abs(C{i}));
end
total=sum(mass);
return


function [Contribs, maxContribs, diffmaxContribs] = reshape_normalize_contribs(C, maxOrder, seqLen, opts)

alphabetSize = 4;
l=0;
for( i = 1:maxOrder )
	L = l + alphabetSize^i * seqLen;
	Contribs{i} = reshape( C((l+1):L), alphabetSize^i, seqLen );
	l = L;
end;
assert( l == length(C) );

if( exist('opts', 'var') )
	if isfield(opts, 'distribution')
		strprobs = getstringprobsMC(length(Contribs), opts.distribution, seqLen, 4);
		MyV2 = getV2_poimMC(Contribs, strprobs, seqLen, 4);   

		for( i = 1:maxOrder )
			Contribs{i} = Contribs{i} -MyV2{i};
		end
	end

	if isfield(opts, 'background')
		for( i = 1:maxOrder )
			Contribs{i} = Contribs{i}.*double(opts.background{i}~=0);
		end
	end
end

maxContribs = zeros( maxOrder, seqLen );
maxp_org = zeros( maxOrder, seqLen );
maxp_str=zeros(maxOrder,seqLen);
for( i = 1:maxOrder )
	[maxContribs(i,:),maxp_str(i,:)]=max(abs((Contribs{i})));
	maxp_org(i,:)=maxContribs(i,:).*sign([Contribs{i}(maxp_str(i,:))]);
	diffmaxContribs = maxContribs;
end;

for( k = 2:maxOrder )
	numsy=4^k;
	for l=1:seqLen-k+1,
		km=maxp_str(k,l)-1;
		A=abs(Contribs{k-1}(floor(km/4)+1,l));
		B=abs(Contribs{k-1}(mod(km,numsy/4)+1,l+1));
		zA=mod(km,4)+1;
		zB=floor(km/(numsy/4))+1;
		%correction=sum([A/distribution(zA, l+k-1), B/distribution(zB, l)]);
		correction=max([A, B]);
		diffmaxContribs(k,l) = maxContribs(k,l) - correction;
	end
end;
return

function [w_hull, w_kmerdiversity] = compute_hull_and_diversity(w, maxOrder)
w_hull=zeros(maxOrder,size(w{1},2));
w_kmerdiversity=cell(1,length(w));

for i=1:maxOrder,
	[spec,hull,idx] = compute_hull(w{i});
	w_hull(i,:)=hull;
	w_kmerdiversity{i}=spec;
end
return

function [new_spec,hull,idx] = compute_hull(specs)
specs=-sort(-abs(specs));
minidx=[];
for i=1:size(specs,2),
	spec=(specs(1:end-1,i)-specs(2:end,i))';
	N=size(spec,2);

	LG = zeros(N, 1); % the logs
	S = spec.^2;
	S1 = cumsum(S);   % the first half
	S2 = fliplr(cumsum(fliplr(S))); % the last half
	N1 = 1:N;  N2 = N:-1:1;

	S1 = S1(1:N-1); S2 = S2(2:N);
	N1 = N1(1:N-1); N2 = N2(2:N);
	I = 1:N-1;

	LG = I.*slog(S1./N1) + (N-I).*slog(S2./N2);
	LG = I.*slog(S1./N1) + (N-I).*slog(S2./N2);
	[v,j]=min(LG);
	minidx=[minidx j];
end
v=sort(minidx);
idx=v(round(0.8*length(minidx)));
hull=minidx;
new_spec=flipud(specs(1:idx,:));
return

function x=slog(y)

x=log(y);
idx=find(y==0);
x(idx)=0;
return

function pmatrix = getstringprobsMC(maxOrder,distrib,len, abcSize)

pmatrix = {};
for k=1:maxOrder
	pmatrix{k} = ones(4^k,len);
end;

for l=1:len %this needs speeding up...
	for sigma=1:abcSize
		prob = distrib(sigma,l);
		for k=1:maxOrder
			for relpos = 1:min(k,l)
				vi = genindexvector_spos(k,sigma-1,relpos,abcSize);
				pmatrix{k}(vi,l-relpos+1) = pmatrix{k}(vi,l-relpos+1)*prob;
			end;
		end;
	end;
end;
return;

function iv = genindexvector_spos(kk,sigma,pos,abcSize)
% get all indices of strings of length kk with sigma at position pos
  iv = zeros(1,abcSize^kk);
  v = [0:(abcSize^kk - 1)];
  r = mod(v,abcSize^(kk-pos+1));
  b= floor(r/abcSize^(kk-pos));
  iv = find(b==sigma);
return

function VV = getV2_poimMC(u,strprobs,len,abcSize)
  VV = {};
  for k=1:length(u)
    m = abcSize^k;
    VV{k} = ones(4^k,1)*mean(u{k}.*strprobs{k});
end;
return;
